`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/03/08 00:33:50
// Design Name: 
// Module Name: freqdiv
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

`define HALF_TARGET_FREQ 26'd50000000 
module freqdiv(
    clk_out, //divided clock output
    clk_ctl, //clock for 7-segment
    clk,    //clock input
    rst_n   //low-active reset
);
    output reg clk_out; //divided clk output(in always block)
    input clk, rst_n; //clock input, low-active reset
    output reg [1:0] clk_ctl; 
    
    reg clk_tmp; //input to the other D-FF
    reg [14:0] cnt_h; //temp buffer for the counter
    reg [8:0] cnt_l; //temp buffer for the counter
    reg [25:0] cnt_tmp_a; //input to selector
    reg [25:0] cnt_tmp_b; //input to D-FF
   
    always @*
        begin
            if (cnt_tmp_a == `HALF_TARGET_FREQ) //combinational logics to select
                begin
                clk_tmp = ~clk_out;
                cnt_tmp_b = 26'd0;
                end
            else
                begin
                clk_tmp = clk_out;
                cnt_tmp_b = cnt_tmp_a;
                end

        cnt_tmp_a = {cnt_l, clk_ctl, cnt_h} + 1'b1; //combinational logics to increment(neglecting overflow)
        end
    
    
    //sequential logics
    always @(posedge clk or negedge rst_n)
    if (~rst_n)
        begin
        clk_out <= 1'b0;
        {cnt_l, clk_ctl, cnt_h} <= 26'd0;
        end
    else
        begin
        clk_out <= clk_tmp; //let the signal pass another D-FF (for toggling)
        {cnt_l, clk_ctl, cnt_h} <= cnt_tmp_b;   //let the signal pass one D-FF
        end
        
endmodule
